//
// (c) 2006 DS Data Systems UK Ltd, All rights reserved.
//
// DS Data Systems and KonaKart and their respective logos, are 
// trademarks of DS Data Systems UK Ltd. All rights reserved.
//
// The information in this document is free software; you can redistribute 
// it and/or modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//

package com.konakart.bl.modules.payment.paypal;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ResourceBundle;

import com.konakart.app.KKConfiguration;
import com.konakart.app.KKEng;
import com.konakart.app.KKException;
import com.konakart.app.NameValue;
import com.konakart.app.Order;
import com.konakart.app.OrderTotal;
import com.konakart.app.PaymentDetails;
import com.konakart.bl.modules.BaseModule;
import com.konakart.bl.modules.ordertotal.OrderTotalMgr;
import com.konakart.bl.modules.payment.BasePaymentModule;
import com.konakart.bl.modules.payment.PaymentInfo;
import com.konakart.bl.modules.payment.PaymentInterface;

/**
 * PayPal IPN module
 */
public class Paypal extends BasePaymentModule implements PaymentInterface
{

    private static final String hostPortSubstitute = "host:port";

    private static int sortOrder = -1;

    private static String payPalId;

    private static String payPalCallbackUrl;

    private static String payPalReturnUrl;

    private static String payPalCancelUrl;

    private static boolean payPalTestMode;

    private static int zone;

    private static int orderStatusId;

    private static String code = "paypal";

    private static String bundleName = BaseModule.basePackage + ".payment.paypal.Paypal";

    private static HashMap resourceBundleMap = new HashMap();

    private static String mutex = "paypalMutex";

    private KKEng eng;

    // Configuration Keys

    private final static String MODULE_PAYMENT_PAYPAL_STATUS = "MODULE_PAYMENT_PAYPAL_STATUS";

    private final static String MODULE_PAYMENT_PAYPAL_ID = "MODULE_PAYMENT_PAYPAL_ID";

    private final static String MODULE_PAYMENT_PAYPAL_ZONE = "MODULE_PAYMENT_PAYPAL_ZONE";

    private final static String MODULE_PAYMENT_PAYPAL_ORDER_STATUS_ID = "MODULE_PAYMENT_PAYPAL_ORDER_STATUS_ID";

    private final static String MODULE_PAYMENT_PAYPAL_SORT_ORDER = "MODULE_PAYMENT_PAYPAL_SORT_ORDER";

    /**
     * This URL is used by the PayPal IPN functionality to call back into the application with the
     * results of the payment transaction. It must be a URl that is visible from the internet.
     */
    private final static String MODULE_PAYMENT_PAYPAL_CALLBACK_URL = "MODULE_PAYMENT_PAYPAL_CALLBACK_URL";

    /**
     * This URL is used by PayPal to redirect the user's browser when returning from the payment
     * gateway. If it is in the form http://host:port/konakart/CheckoutFinished.do, then the string
     * host:port is substituted automatically with the correct value.
     */
    private final static String MODULE_PAYMENT_PAYPAL_RETURN_URL = "MODULE_PAYMENT_PAYPAL_RETURN_URL";

    /**
     * This URL is used by PayPal to redirect the user's browser when returning from the payment
     * gateway after cancelling out of the operation. If it is in the form
     * http://host:port/konakart/CatalogCheckoutExternalPaymentErrorPage.do, then the string
     * host:port is substituted automatically with the correct value.
     */
    private final static String MODULE_PAYMENT_PAYPAL_CANCEL_URL = "MODULE_PAYMENT_PAYPAL_CANCEL_URL";

    /**
     * If set to true, the module will use the PayPal sandbox. Otherwise the live URL will be used.
     */
    private final static String MODULE_PAYMENT_PAYPAL_TEST_MODE = "MODULE_PAYMENT_PAYPAL_TEST_MODE";

    // Message Catalogue Keys

    private final static String MODULE_PAYMENT_PAYPAL_TEXT_TITLE = "module.payment.paypal.text.title";

    private final static String MODULE_PAYMENT_PAYPAL_TEXT_DESCRIPTION = "module.payment.paypal.text.description";

    /**
     * Constructor
     * 
     * @throws KKException
     */
    public Paypal() throws KKException
    {
        // Instantiate an object to interface to the main engine
        eng = new KKEng();

        // Initialise the static variables
        if (sortOrder == -1)
        {
            synchronized (mutex)
            {
                if (sortOrder == -1)
                {
                    setStaticVariables();
                }
            }
        }
    }

    /**
     * Sets some static variables during setup
     * 
     * @throws KKException
     * 
     */
    public void setStaticVariables() throws KKException
    {
        KKConfiguration conf;

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_ID);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_PAYPAL_ID must be set to the PayPal Id of the merchant.");
        }
        payPalId = conf.getValue();

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_CALLBACK_URL);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_PAYPAL_CALLBACK must be set to the Callback Url for the"
                            + " IPN functionality (i.e. https://myhost/konakart/PayPalCallback.do).");
        }
        payPalCallbackUrl = conf.getValue();

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_RETURN_URL);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_PAYPAL_RETURN_URL must be set to the return URL for"
                            + " when the customer leaves the payment gateway. (i.e. http://{host:port}/konakart/CheckoutFinished.do)");
        }
        payPalReturnUrl = conf.getValue();

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_CANCEL_URL);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_PAYPAL_CANCEL_URL must be set to the return URL for"
                            + " when the customer leaves the payment gateway by cancelling the operation. (i.e. http://{host:port}/konakart/CatalogCheckoutExternalPaymentErrorPage.do)");
        }
        payPalCancelUrl = conf.getValue();

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_ZONE);
        if (conf == null)
        {
            zone = 0;
        } else
        {
            zone = new Integer(conf.getValue()).intValue();
        }

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_SORT_ORDER);
        if (conf == null)
        {
            sortOrder = 0;
        } else
        {
            sortOrder = new Integer(conf.getValue()).intValue();
        }

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_ORDER_STATUS_ID);
        if (conf == null)
        {
            orderStatusId = 0;
        } else
        {
            orderStatusId = new Integer(conf.getValue()).intValue();
        }

        conf = eng.getConfiguration(MODULE_PAYMENT_PAYPAL_TEST_MODE);
        if (conf == null)
        {
            payPalTestMode = true;
        } else
        {
            if (conf.getValue().equalsIgnoreCase("false"))
            {
                payPalTestMode = false;
            } else
            {
                payPalTestMode = true;
            }
        }

    }

    @SuppressWarnings("unchecked")
    /**
     * Return a payment details object for PayPal IPN module
     * 
     * @param order
     * @param info
     * @return Returns information in a PaymentDetails object
     * @throws Exception
     */
    public PaymentDetails getPaymentDetails(Order order, PaymentInfo info) throws Exception
    {

        /*
         * The payPalZone zone, if greater than zero, should reference a GeoZone. If the
         * DeliveryAddress of the order isn't within that GeoZone, then we throw an exception
         */
        if (zone > 0)
        {
            checkZone(info, zone);
        }

        // Get the scale for currency calculations
        int scale = new Integer(order.getCurrency().getDecimalPlaces()).intValue();

        // Get the resource bundle
        ResourceBundle rb = getResourceBundle(mutex, bundleName, resourceBundleMap, info
                .getLocale());
        if (rb == null)
        {
            throw new KKException("A resource file cannot be found for the country "
                    + info.getLocale().getCountry());
        }

        PaymentDetails pDetails = new PaymentDetails();
        pDetails.setCode(code);
        pDetails.setSortOrder(sortOrder);
        pDetails.setOrderStatusId(orderStatusId);
        pDetails.setPaymentType(PaymentDetails.BROWSER_PAYMENT_GATEWAY);
        pDetails.setDescription(rb.getString(MODULE_PAYMENT_PAYPAL_TEXT_DESCRIPTION));
        pDetails.setTitle(rb.getString(MODULE_PAYMENT_PAYPAL_TEXT_TITLE));

        // Return now if the full payment details aren't required
        if (info.isReturnDetails() == false)
        {
            return pDetails;
        }

        pDetails.setPostOrGet("post");
        if (payPalTestMode)
        {
            pDetails.setRequestUrl("https://www.sandbox.paypal.com/cgi-bin/webscr");
        } else
        {
            pDetails.setRequestUrl("https://www.paypal.com/cgi-bin/webscr");
        }

        List parmList = new ArrayList();
        parmList.add(new NameValue("cmd", "_xclick"));
        parmList.add(new NameValue("item_name", "Order #" + order.getId() + " from "
                + info.getStoreName()));

        BigDecimal total = null;
        for (int i = 0; i < order.getOrderTotals().length; i++)
        {
            OrderTotal ot = (OrderTotal) order.getOrderTotals()[i];
            if (ot.getClassName().equals(OrderTotalMgr.ot_total))
            {
                total = ot.getValue().setScale(scale, BigDecimal.ROUND_HALF_UP);
            }
        }

        if (total == null)
        {
            throw new KKException(
                    "An Order Total was not found so the payment could not be processed through PayPal.");
        }

        parmList.add(new NameValue("amount", total.toString()));
        parmList.add(new NameValue("business", payPalId));
        parmList.add(new NameValue("currency_code", order.getCurrencyCode()));
        parmList.add(new NameValue("custom", info.getSecretKey()));
        parmList.add(new NameValue("no_shipping", "1"));
        parmList.add(new NameValue("no_note", "1"));
        parmList.add(new NameValue("notify_url", payPalCallbackUrl));

        payPalReturnUrl = payPalReturnUrl.replaceFirst(hostPortSubstitute, info.getHostAndPort());
        payPalCancelUrl = payPalCancelUrl.replaceFirst(hostPortSubstitute, info.getHostAndPort());
        parmList.add(new NameValue("return", payPalReturnUrl));
        parmList.add(new NameValue("cancel_return", payPalCancelUrl));

        // Put the parameters into an array
        NameValue[] nvArray = new NameValue[parmList.size()];
        parmList.toArray(nvArray);
        pDetails.setParameters(nvArray);

        if (log.isDebugEnabled())
        {
            log.debug(pDetails.toString());
        }

        return pDetails;
    }

    /**
     * Returns true or false
     * 
     * @throws KKException
     */
    public boolean isAvailable() throws KKException
    {
        return isAvailable(eng, MODULE_PAYMENT_PAYPAL_STATUS);
    }
}
